from pygame import Rect, draw
from widget import Widget
import resource

class Label(Widget):

	default_fg_color = (255, 255, 255)
	
	def __init__(self, text, font = None, width = None, fg_color = None):
		if fg_color is not None:
			self.fg_color = fg_color
		font = font or resource.get_default_font()
		lines = text.split("\n")
		tw, th = 0, 0
		for line in lines:
			w, h = font.size(line)
			tw = max(tw, w)
			th += h
		if width is not None:
			tw = width
		else:
			tw = max(1, tw)
		Widget.__init__(self, Rect((0, 0), (tw, th)))
		self.text = text
		self.font = font

	def draw(self, surface):
		self.draw_with(surface, self.get_fg_color())
	
	def draw_with(self, surface, fg):
		y = 0
		lines = self.text.split("\n")
		font = self.font
		dy = font.get_linesize()
		for line in lines:
			image = font.render(line, True, fg)
			surface.blit(image, (0, y))
			y += dy

#---------------------------------------------------------------------------

class Button(Label):

	fg_color = (255, 255, 0)
	highlight_color = (255, 0, 0)
	disabled_color = (64, 64, 64)
	
	highlighted = False
	_enabled = True
	
	def __init__(self, text, font = None, action = None, enable = None):
		font = font or resource.get_font(15, "VeraBd.ttf")
		Label.__init__(self, text, font)
		if action:
			self.action = action
		if enable:
			self.get_enabled = enable
	
	def draw(self, surface):
		if self.enabled:
			if self.highlighted:
				color = self.highlight_color
			else:
				color = self.fg_color
		else:
			color = self.disabled_color
		self.draw_with(surface, color)
	
	def mouse_down(self, event):
		if self.enabled:
			self.highlighted = True
	
	def mouse_up(self, event):
		if self.highlighted:
			self.highlighted = False
			self.call_handler('action')
	
	def action(self):
		pass
	
	def get_enabled(self):
		return self._enabled
	
	def set_enabled(self, x):
		self._enabled = x
	
	enabled = property(
		lambda self: self.get_enabled(),
		lambda self, x: self.set_enabled(x)
	)

#---------------------------------------------------------------------------

class TextField(Widget):

	fg_color = (0, 255, 0)

	def __init__(self, width, font = None):
		font = font or resource.get_default_font()
		Widget.__init__(self, Rect(0, 0, width, font.get_height())) #linesize()))
		self.font = font
		self.text = ""
	
	def get_text(self):
		return self.text
	
	def set_text(self, text):
		self.text = text
	
	def draw(self, surface):
		fg = self.get_fg_color()
		image = self.font.render(self.get_text(), True, fg)
		surface.blit(image, (0, 0))
		if self.has_focus():
			r = image.get_rect()
			draw.line(surface, fg, r.topright, r.bottomright)
	
	def key_down(self, event):
		try:
			c = event.unicode
		except ValueError:
			c = ""
		if self.insert_char(c) == 'pass':
			self.call_parent_handler('key_down', event)
	
	def insert_char(self, c):
		if c <= "\x7f":
			if c == "\x08" or c == "\x7f":
				self.change_text(self.get_text()[:-1])
				return
			elif c == "\r" or c == "\x03":
				if self.call_handler('enter_action'):
					return
			elif c == "\x1b":
				if self.call_handler('escape_action'):
					return
			elif c >= "\x20":
				self.change_text(self.get_text() + c)
				return
		return 'pass'

	def mouse_down(self, event):
		self.focus()

	def change_text(self, text):
		self.set_text(text)
		self.call_handler('change_action')

#---------------------------------------------------------------------------

class RowOrColumn(Widget):

	def __init__(self, size, items, kwds):
		align = kwds.pop('align', 'c')
		s = kwds.pop('spacing', 10)
		expand = kwds.pop('expand', None)
		Widget.__init__(self, Rect(0, 0, 100, 100))
		d = self.d
		longways = self.longways
		crossways = self.crossways
		k, attr2, attr3 = self.align_map[align]
		w = 0
		l = 0
		for item in items:
			r = item.rect
			w = max(w, getattr(r, crossways))
			l += getattr(r, longways)
		if size is not None:
			expand = expand or items[-1]
			n = len(items)
			if n > 1:
				l += s * (n - 1)
			setattr(expand.rect, longways, max(1, size - l))
		h = w * k // 2
		px = h * d[1]
		py = h * d[0]
		sx = s * d[0]
		sy = s * d[1]
		for item in items:
			setattr(item.rect, attr2, (px, py))
			self.add(item)
			p = getattr(item.rect, attr3)
			px = p[0] + sx
			py = p[1] + sy
		self.shrink_wrap()

class Row(RowOrColumn):

	d = (1, 0)
	longways = 'width'
	crossways = 'height'
	align_map = {
		't': (0, 'topleft', 'topright'),
		'c': (1, 'midleft', 'midright'),
		'b': (2, 'bottomleft', 'bottomright'),
	}

	def __init__(self, items, width = None, **kwds):
		"""
		Row(items, align = alignment, spacing = 10, width = None, expand = None)
		align = 't', 'c' or 'b'
		"""
		RowOrColumn.__init__(self, width, items, kwds)

class Column(RowOrColumn):

	d = (0, 1)
	longways = 'height'
	crossways = 'width'
	align_map = {
		'l': (0, 'topleft', 'bottomleft'),
		'c': (1, 'midtop', 'midbottom'),
		'r': (2, 'topright', 'bottomright'),
	}

	def __init__(self, items, height = None, **kwds):
		"""
		Column(items, align = alignment, spacing = 10, height = None, expand = None)
		align = 'l', 'c' or 'r'
		"""
		RowOrColumn.__init__(self, height, items, kwds)

#---------------------------------------------------------------------------

class Frame(Widget):
	#  border_spacing  int        spacing between border and widget

	border_width = 1
	border_spacing = 2

	def __init__(self, client, border_width = None, border_spacing = None):
		self.client = client
		if border_width is not None:
			self.border_width = border_width
		if border_spacing is not None:
			self.border_spacing = border_spacing
		d = self.border_width + self.border_spacing
		f = client.rect.inflate(2 * d, 2 * d)
		Widget.__init__(self, Rect((0, 0), f.size))
		client.rect.topleft = (d, d)
		self.add(client)

#---------------------------------------------------------------------------

class Image(Widget):
	#  image   Image to display

	def __init__(self, image):
		Widget.__init__(self, image.get_rect())
		self.image = image
	
	def draw(self, surf):
		surf.blit(self.image, (0, 0))
