#
#   Albow - File Dialogs
#

import os
from pygame import draw, Rect
from albow.widget import Widget
from albow.dialogs import Dialog, ask
from albow.controls import Label, Button, TextField, Row, Column, Frame
from albow.palette_view import PaletteView
from albow.resource import get_default_font

class DirPathView(Widget):

	font = get_default_font()

	def __init__(self, width, client):
		self.client = client
		Widget.__init__(self, Rect(0, 0, width, self.font.get_linesize()))
	
	def draw(self, surf):
		image = self.font.render(self.client.directory, True, self.get_fg_color())
		tw = image.get_width()
		mw = surf.get_width()
		if tw <= mw:
			x = 0
		else:
			x = mw - tw
		surf.blit(image, (x, 0))


class FileListView(PaletteView):

	font = get_default_font()
	scroll_button_color = (255, 255, 0)

	def __init__(self, width, client):
		h = self.get_font().get_linesize()
		PaletteView.__init__(self, (width, h), 10, 1, scrolling = True)
		self.client = client
		self.selection = None
		self.names = []
	
	def update(self):
		client = self.client
		dir = client.directory
		suffixes = client.suffixes
		def filter(name):
			path = os.path.join(dir, name)
			if os.path.isdir(path):
				return True
			if not suffixes:
				return True
			for suffix in suffixes:
				if name.endswith(suffix):
					return True
			return False
		self.names = [name for name in os.listdir(client.directory)
			if not name.startswith(".") and filter(name)]
		self.selection = None
	
	def num_items(self):
		return len(self.names)
	
	def draw_prehighlight(self, surf, item_no, rect):
		draw.rect(surf, self.sel_color, rect)
	
	def draw_item(self, surf, item_no, rect):
		font = self.get_font()
		color = self.get_fg_color()
		buf = self.font.render(self.names[item_no], True, color)
		surf.blit(buf, rect)
	
	def click_item(self, item_no, e):
		self.selection = item_no
		self.client.dir_box_click(e.num_clicks == 2)
	
	def item_is_selected(self, item_no):
		return item_no == self.selection
	
	def get_selected_name(self):
		sel = self.selection
		if sel is not None:
			return self.names[sel]
		else:
			return ""
		

class FileDialog(Dialog):

	box_width = 250
	fg_color = (255, 255, 255)
	border_width = 1

	def __init__(self, prompt = None, suffixes = None):
		Dialog.__init__(self)
		self.suffixes = suffixes
		up_button = Button("<-", action = self.go_up)
		dir_box = DirPathView(self.box_width - up_button.rect.width, self)
		self.dir_box = dir_box
		top_row = Row([dir_box, up_button])
		list_box = FileListView(self.box_width - 16, self)
		self.list_box = list_box
		ctrls = [top_row, Frame(list_box)]
		if self.saving:
			label = Label(prompt or self.default_prompt)
			filename_box = TextField(self.box_width)
			filename_box.fg_color = None
			filename_box.change_action = self.update
			self.filename_box = filename_box
			ctrls.append(label)
			ctrls.append(Frame(filename_box))
		ok_button = Button(self.ok_label, action = self.ok, enable = self.ok_enable)
		self.ok_button = ok_button
		cancel_button = Button("Cancel", action = self.cancel)
		vbox = Column(ctrls, align = 'l', spacing = 10)
		vbox.rect.topleft = (20, 20)
		y = vbox.rect.bottom + 10
		ok_button.rect.topleft = (vbox.rect.left, y)
		cancel_button.rect.topright = (vbox.rect.right, y)
		self.add(vbox)
		self.add(ok_button)
		self.add(cancel_button)
		self.shrink_wrap()
		self._directory = None
		self.directory = os.getcwd()
	
	def get_directory(self):
		return self._directory
	
	def set_directory(self, x):
		if self._directory <> x:
			self._directory = x
			#self.dir_box.text = x
			self.list_box.update()
			self.update()
	
	def ok_enable(self):
		return True
	
	directory = property(get_directory, set_directory)
	
	def update(self):
		pass
	
	def go_up(self):
		self.directory = os.path.dirname(self.directory)
	
	def dir_box_click(self, double):
		if double:
			name = self.list_box.get_selected_name()
			path = os.path.join(self.directory, name)
			if os.path.isdir(path):
				self.directory = path
			else:
				self.double_click_file(name)
		self.update()
	
	def ok(self):
		self.dismiss(True)
	
	def cancel(self):
		self.dismiss(False)
	

class FileSaveDialog(FileDialog):

	saving = True
	default_prompt = "Save as:"
	ok_label = "Save"
	
	def get_filename(self):
		return self.filename_box.get_text()
	
	def set_filename(self, x):
		dsuf = self.suffixes[0]
		if x.endswith(dsuf):
			x = x[:-len(dsuf)]
		self.filename_box.set_text(x)
	
	filename = property(get_filename, set_filename)

	def get_pathname(self):
		path = os.path.join(self.directory, self.filename_box.get_text())
		suffixes = self.suffixes
		if suffixes and not path.endswith(suffixes[0]):
			path = path + suffixes[0]
		return path
	
	pathname = property(get_pathname)
	
	def double_click_file(self, name):
		self.filename_box.set_text(name)

	def ok(self):
		path = self.pathname
		if os.path.exists(path):
			answer = ask("Replace existing '%s'?" % os.path.basename(path))
			if answer <> "OK":
				return
		FileDialog.ok(self)

	def update(self):
		FileDialog.update(self)
		name = self.filename_box.get_text()
		#self.ok_button.enabled = name <> ""
	
	def ok_enable(self):
		return self.filename_box.get_text() <> ""


class FileOpenDialog(FileDialog):

	saving = False
	ok_label = "Open"
	
	def get_pathname(self):
		return os.path.join(self.directory, self.list_box.get_selected_name())

	pathname = property(get_pathname)
	
	def update(self):
		FileDialog.update(self)
		path = self.pathname
		self.ok_button.enabled = path and os.path.isfile(path)
	
	def double_click_file(self, name):
		self.ok()


def request_new_filename(prompt = None, suffix = None, extra_suffixes = None,
		directory = None, filename = None, pathname = None):
	if pathname:
		directory, filename = os.path.split(pathname)
	if extra_suffixes:
		suffixes = extra_suffixes
	else:
		suffixes = []
	if suffix:
		suffixes = [suffix] + suffixes
	dlog = FileSaveDialog(prompt = prompt, suffixes = suffixes)
	if directory:
		dlog.directory = directory
	if filename:
		dlog.filename = filename
	if dlog.present():
		return dlog.pathname
	else:
		return None

def request_old_filename(suffixes = None, directory = None):
	dlog = FileOpenDialog(suffixes = suffixes)
	if directory:
		dlog.directory = directory
	if dlog.present():
		return dlog.pathname
	else:
		return None
