#
#   5B - Game Shell
#

import os
from pygame import Rect
from albow.widget import Widget
from albow.shell import Shell
from albow.controls import Label, Button, Column, Image
from albow.dialogs import alert, ask
from albow.file_dialogs import request_old_filename, request_new_filename
from albow.resource import get_font, get_image
from screen import Screen, menu_button
from play import LevelScreen
from editor import EditorMenuScreen, EditorLevelScreen
from game import Game
from level import Level
import settings
from settings import options

class FBShell(Shell):

	def __init__(self, display):
		Shell.__init__(self, display)
		self.game = Game()
		self.menu_screen = MenuScreen(self)
		self.story_screen = StoryScreen(self)
		self.level_screen = LevelScreen(self)
		self.editor_menu_screen = EditorMenuScreen(self)
		self.editor_level_screen = EditorLevelScreen(self)
		self.set_timer(settings.frame_time)
		level_file_passed = self.handle_filename_option(options.filename)
		if options.edit:
			if not self.game.current_level:
				self.edit_new_level()
			else:
				self.show_editor_level()
		else:
			if level_file_passed:
				self.start_level()
			else:
				self.show_menu()
	
	def handle_filename_option(self, path):
		if path:
			if os.path.isdir(path):
				self.game.current_level_dir = path
			elif path.endswith(Level.level_file_suffix):
				self.game.current_level_dir = os.path.dirname(path)
				self.load_level_file(path)
				print "shell: handle_filename_option: current_level_dir =", \
					self.game.current_level_dir ###
				return True
			else:
				alert("%s is not a directory or recognised file type."
					% path)
	
	def show_menu(self):
		self.show_screen(self.menu_screen)
	
	def show_story(self):
		self.show_screen(self.story_screen)
	
	def show_level(self):
		self.show_screen(self.level_screen)
	
	def new_game(self):
		paths = self.game.get_current_playing_level_list()
		if not paths:
			alert("There are no levels in %s." % 
				self.game.get_current_playing_level_dir())
			return
		self.load_file(paths[0])
		self.start_level()
	
	def start_level(self):
		self.game.start_level()
		self.show_level()
	
	def next_level(self):
		path = self.game.get_next_level_path()
		if path:
			self.load_level_file(path)
			self.start_level()
		else:
			self.show_menu()
	
	def enter_editor(self):
		self.game.abort_level()
		self.show_screen(self.editor_menu_screen)
	
	def show_editor_menu(self):
		self.show_screen(self.editor_menu_screen)
	
	def show_editor_level(self):
		self.show_screen(self.editor_level_screen)
	
	def edit_new_level(self):
		if not self.ask_save_level():
			return
		self.game.create_level()
		self.show_editor_level()
	
	def quit(self):
		if self.ask_save_level():
			Shell.quit(self)
	
	def load_file(self, path):
		if path.endswith(Level.level_file_suffix):
			self.load_level_file(path)
		else:
			alert("%s is not a recognised file type." % path)
	
	def load_level(self):
		if not self.ask_save_level():
			return
		directory = self.game.get_current_custom_level_dir()
		path = request_old_filename(suffixes = [Level.level_file_suffix],
			directory = directory)
		if path:
			self.game.current_level_dir = os.path.dirname(path)
			self.load_level_file(path)
	
	def load_level_file(self, path):
		try:
			self.game.load_level(path)
		except EnvironmentError, e:
			alert("Unable to load level. %s" % e)
	
	def save_level(self):
		level = self.game.current_level
		old_path = level.pathname
		if old_path:
			old_dir, old_name = os.path.split(old_path)
		else:
			old_dir = self.game.get_current_custom_level_dir()
			old_name = None
		path = request_new_filename(prompt = "Save Level As:",
			suffix = Level.level_file_suffix,
			directory = old_dir, filename = old_name)
		if path:
			level.pathname = path
			try:
				self.game.save_level(path)
			except EnvironmentError, e:
				alert("Unable to save level. %s" % e)
			return True
		else:
			return False
	
	def ask_save_level(self):
		level = self.game.current_level
		if level and level.changed:
			name = level.get_name()
			response = ask("Save changes to '%s'?" % name,
				["Save", "Discard", "Cancel"])
			if response == "Cancel":
				return False
			if response == "Save":
				if not self.save_level():
					return False
		return True
	
	def begin_frame(self):
		screen = self.current_screen
		if screen:
			self.current_screen.begin_frame()


class MenuScreen(Screen):

	def __init__(self, shell):
		Screen.__init__(self, shell)
		game = shell.game
		mb = menu_button
		items = [
			Image(get_image("title.png")),
			mb("About",           shell.show_story),
			mb("New Game",        shell.new_game),
			mb("Continue Game",   shell.start_level,       game.start_enabled),
			mb("Resume Level",    shell.show_level,        game.resume_enabled),
			mb("Abort Level",     game.abort_level,        game.abort_enabled),
			mb("Quit",            shell.quit),
			Label(""),
			mb("Level Editor", shell.enter_editor),
		]
		menu = Column(items, align = 'l')
		menu.rect.center = self.rect.center
		self.add(menu)


class StoryScreen(Screen):

	def __init__(self, shell):
		Screen.__init__(self, shell)
		items = [
			Image(get_image("story.png")),
			menu_button("Back", action = shell.show_menu),
		]
		box = Column(items, align = 'c', spacing = 30)
		box.rect.center = self.rect.center
		self.add(box)
