#
#   5B - Vectors, matrices and coordinate transformations
#

from __future__ import division
from math import pi

deg = pi / 180

def add2((x1, y1), (x2, y2)):
	return x1 + x2, y1 + y2

def sub2((x1, y1), (x2, y2)):
	return x1 - x2, y1 - y2

class Coords2(object):
	#  coords  (number, number)

	def __init__(self, *coords):
		if len(coords) == 1:
			coords = tuple(coords[0])
		self.coords = coords

	def __iter__(self):
		return iter(self.coords)

	def __eq__(self, other):
		return self.coords == other.coords
	
	def __ne__(self, other):
		return self.coords <> other.coords

	def snap(self, grid):
		def r(z):
			return int(round(z / grid) * grid)
		x, y = self.coords
		return self.__class__(r(x), r(y))

	def __repr__(self):
		x, y = self.coords
		return "%s(%s, %s)" % (self.__class__.__name__, x, y)


class Vector2(Coords2):

	def __add__(self, other):
		if isinstance(other, Vector2):
			return Vector2(add2(self.coords, other.coords))
		return NotImplemented
	
	def __sub__(self, other):
		if isinstance(other, Vector2):
			return Vector2(sub2(self.coords, other.coords))
		return NotImplemented


Zero2 = Vector2(0, 0)


class Point2(Coords2):

	def __add__(self, other):
		if isinstance(other, Vector2):
			return Point2(add2(self.coords, other.coords))
		return NotImplemented
	
	__radd__ = __add__
	
	def __sub__(self, other):
		if isinstance(other, Point2):
			return Vector2(sub2(self.coords, other.coords))
		elif isinstance(other, Vector2):
			return Point2(sub2(self.coords, other.coords))
		return NotImplemented
	
	def aligns_with(self, *others):
		h = True
		v = True
		x1, y1 = self.coords
		for other in others:
			x2, y2 = other.coords
			if x1 <> x2:
				v = False
			if y1 <> y2:
				h = False
		return h or v


Origin2 = Point2(0, 0)


class Matrix2(object):
	#  coords   ((number, number), (number, number))
	
	def __init__(self, coords):
		self.coords = coords
	
	def __repr__(self):
		return "Matrix2%r" % (self.coords,)
	
	def __iter__(self):
		return iter(self.coords)

	def __mul__(self, other):
		if isinstance(other, Coords2):
			(a11, a12), (a21, a22) = self.coords
			u1, u2 = other.coords
			v1 = a11 * u1 + a12 * u2
			v2 = a21 * u1 + a22 * u2
			return other.__class__(v1, v2)
		if isinstance(other, Matrix2):
			(a11, a12), (a21, a22) = self.coords
			(b11, b12), (b21, b22) = other.coords
			c11 = a11 * b11 + a12 * b21
			c12 = a11 * b12 + a12 * b22
			c21 = a21 * b11 + a22 * b21
			c22 = a21 * b12 + a22 * b22
			return Matrix2(((c11, c12), (c21, c22)))
		return NotImplemented
	
	def get_transpose(self):
		(a11, a12), (a21, a22) = self.coords
		return Matrix2(((a11, a21), (a12, a22)))


IdentMat2 = Matrix2(((1, 0), (0, 1)))
FlipXMat2 = Matrix2(((-1, 0), (0, 1)))
FlipYMat2 = Matrix2(((1, 0), (0, -1)))
RotCCWMat2 = Matrix2(((0, 1), (-1, 0)))  #  90 deg counter clockwise assuming y is down

def ScaleMat2(s):
	return Matrix2(((s, 0), (0, s)))

OrientMat2 = [
	IdentMat2, RotCCWMat2,
	FlipXMat2, FlipXMat2 * RotCCWMat2,
	FlipYMat2, FlipYMat2 * RotCCWMat2,
	FlipYMat2 * FlipXMat2, FlipYMat2 * FlipXMat2 * RotCCWMat2
]


class Transformation2(object):
	#  matrix       Matrix2
	#  vector       Vector2
	#  _transpose   Transformation2

	def __init__(self, matrix, vector):
		self.matrix = matrix
		self.vector = vector
		self._transpose = None
		self.origin = Point2(vector.coords)

	def __repr__(self):
		return "Transformation2(%r,%r)" % (self.matrix, self.vector)
	
	def __mul__(self, other):
		if isinstance(other, Vector2):
			return self.matrix * other
		if isinstance(other, Point2):
			return self.matrix * other + self.vector
		if isinstance(self, Transformation2):
			return Transformation2(self.matrix * other.matrix,
				self.matrix * other.vector + self.vector)
	
	def get_transpose(self):
		result = self._transpose
		if not result:
			m = self.matrix.get_transpose()
			v = - m * self.vector
			result = Transformation2(m, v)
			self._transpose = result
		return result


IdentTran2 = Transformation2(IdentMat2, Zero2)
FlipXTran2 = Transformation2(FlipXMat2, Zero2)
FlipYTran2 = Transformation2(FlipYMat2, Zero2)
RotCCWTran2 = Transformation2(RotCCWMat2, Zero2)

def Translation2(vector):
	return Transformation2(IdentMat2, vector)

def ScaleTran2(s):
	return Transformation2(ScaleMat2(s), Zero2)

OrientTran2 = [
	Transformation2(OrientMat2[i], Zero2)
		for i in xrange(8)]
